/*	
  Copyright(c) 2011 Christoph Schmalz <christoph.schmalz@informatik.uni-erlangen.de>
  
  This file is licensed under the terms of of the GNU General Public License, 
  version 3, as published by the Free Software Foundation. You can find it at:
  http://www.gnu.org/licenses/gpl.html
*/

#include "stdafx.h"
#include "phaseshiftpattern.h"
#include "util.h"

///////////////////////////////////////////////////////////////////////////////
// Function: CPhaseShiftPattern::DrawGrayCode
///////////////////////////////////////////////////////////////////////////////

void CPhaseShiftPattern::DrawGrayCode( 
	Mat&		img,
	int			iSlideNr,
	int			iTotalSlides,
	bool		bInvert) const
{
	// Of course the pattern has to be valid.
	if (!IsValid() || iSlideNr<0 || iSlideNr>=iTotalSlides) 
		return;

	img = Mat( m_iPatternHeight, m_iPatternWidth, CV_8UC1 );

	const unsigned int	uiMaxId			= (1<<iTotalSlides)-1;
	const int			iStripeWidth	= m_iWaveLength/2;

	// Inverting simply implies swapping fore- and background color
	const float			fFGCol		= (bInvert ? 0.f : 255.f);
	const float			fBGCol		= (bInvert ? 255.f : 0.f);

	const int			primarySize		= AreStripesHor() ? 
											m_iPatternHeight : m_iPatternWidth;
	const int			secondarySize	= AreStripesHor() ? 
											m_iPatternWidth : m_iPatternHeight;

	// Fill with background color
	img.setTo( Scalar::all(fBGCol) );

	// Now loop over the whole area
	for (int i=0; i<primarySize; i+=iStripeWidth)
	{
		unsigned int	uiStripeId	=i/iStripeWidth;
		unsigned int	uiGraycode;

		uiGraycode= BinaryToGrayCode(uiStripeId&uiMaxId);

		// Get the relevant bit
		unsigned int uiCurBit = (uiGraycode>>(iTotalSlides-1-iSlideNr));

		// If it is set, draw the stripe in the foreground color
		if (uiCurBit&1)
		{
			if (AreStripesHor())
				rectangle( img, Point(0, i), Point(secondarySize, i+iStripeWidth-1), Scalar::all(fFGCol), -1);
			else
				rectangle( img, Point(i, 0), Point(i+iStripeWidth-1, secondarySize), Scalar::all(fFGCol), -1);
		}
	}
}

///////////////////////////////////////////////////////////////////////////////
// Function: CPhaseShiftPattern::DrawPhaseShiftPattern
///////////////////////////////////////////////////////////////////////////////

void CPhaseShiftPattern::DrawPhaseShiftPattern( 
  Mat&		img,				//in: image to draw to
  double	dPhaseOffset)		//in: phase offset in radians!
  const
{
	assert( dPhaseOffset<=M_2PI );

	img = Mat( m_iPatternHeight, m_iPatternWidth, CV_8UC1 );

	const double		dFGcolor = 255;
	const double		dBGcolor = 0;
	const double		dInvGamma = 1. / m_dGamma;
	int					iStart,iEnd,iStep,iTotalSteps;
	bool				bHorizontal;
	
	switch (m_stripeOrder)
	{
	case STRIPE_ORDER_LEFT_RIGHT:
		iTotalSteps=m_iPatternWidth;
		iStart = 0; iEnd=m_iPatternWidth; iStep=1; bHorizontal=false;
		break;
	case STRIPE_ORDER_TOP_DOWN:
		iTotalSteps=m_iPatternHeight;
		iStart = 0; iEnd = m_iPatternHeight; iStep=1; bHorizontal=true;
		break;
	default:
		assert(false);
		return;
	}

	for (int j=0;j<iTotalSteps;j++)
	{
		const int i = iStart+j*iStep;
		double dLocalPhase = (i%m_iWaveLength) / (double)m_iWaveLength * M_2PI;

		//value of cos in [0;1]
		double dValue = (cos( dLocalPhase-dPhaseOffset ) + 1) * 0.5;

		//apply gamma
		double dCorrectedValue = pow(dValue, dInvGamma);

		//mix foreground and background accordingly
		double dColor = dFGcolor*dCorrectedValue + 
						dBGcolor*(1-dCorrectedValue);

		//mind that there are small quantization errors here
		//as we have to represent the brightness in 8 bits.
		//however, the errors partially cancel between the
		//different patterns, so their influence on the final
		//mark positions can be neglected.

		if (bHorizontal)
			line( img, Point(0,i), Point(m_iPatternWidth,i), Scalar::all(dColor) );
		else
			line( img, Point(i,0), Point(i,m_iPatternHeight), Scalar::all(dColor) );
	}
}
